<?php
/**
 * @package     CSVI
 * @subpackage  VirtueMart
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * The VirtueMart helper class.
 *
 * @package     CSVI
 * @subpackage  VirtueMart
 * @since       6.0
 */
class Com_VirtuemartHelperCom_Virtuemart
{
	/**
	 * Template helper
	 *
	 * @var    CsviHelperTemplate
	 * @since  6.0
	 */
	protected $template = null;

	/**
	 * Logger helper
	 *
	 * @var    CsviHelperLog
	 * @since  6.0
	 */
	protected $log = null;

	/**
	 * Fields helper
	 *
	 * @var    CsviHelperFields
	 * @since  6.0
	 */
	protected $fields = null;

	/**
	 * Database connector
	 *
	 * @var    JDatabaseDriver
	 * @since  6.0
	 */
	protected $db = null;

	/**
	 * The vendor ID
	 *
	 * @var    int
	 * @since  6.0
	 */
	private $vendorId = null;

	/**
	 * Product relation ID
	 *
	 * @var    int
	 * @since  6.0
	 */
	private $relatedId = null;

	/**
	 * Product related field type
	 *
	 * @var    string
	 * @since  7.1.0
	 */
	private $relatedType = null;

	/**
	 * Category separator
	 *
	 * @var    string
	 * @since  6.0
	 */
	private $categorySeparator = null;

	/**
	 * Holds the HTML select list options
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $selectOptions = array();

	/**
	 * The ICEcat helper
	 *
	 * @var    CsviHelperIcecat
	 * @since  6.0
	 */
	private $icecat = null;

	/**
	 * List of processed categories
	 *
	 * @var    array
	 * @since  7.17.0
	 */
	private $categories = [];

	/**
	 * Constructor.
	 *
	 * @param   CsviHelperTemplate  $template  An instance of CsviHelperTemplate.
	 * @param   CsviHelperLog       $log       An instance of CsviHelperLog.
	 * @param   CsviHelperFields    $fields    An instance of CsviHelperFields.
	 * @param   JDatabaseDriver     $db        Database connector.
	 *
	 * @since   4.0
	 */
	public function __construct(CsviHelperTemplate $template, CsviHelperLog $log, CsviHelperFields $fields, JDatabaseDriver $db)
	{
		$this->template     = $template;
		$this->log          = $log;
		$this->fields       = $fields;
		$this->db           = $db;
		$this->icecatFields = array(
			'PROD_ID' => 'related_products',
			'PROD_NAME' => 'product_name',
			'PROD_HIGHPIC' => 'file_url',
			'PROD_THUMBPIC' => 'file_url_thumb',
			'PROD_RELEASEDATE' => 'product_available_date',
			'PROD_CATEGORY_PATH' => 'category_path',
			'PROD_LONGDESC' => 'product_desc',
			'PROD_SHORTDESC' => 'product_s_desc',
			'PROD_MANUFACTURER_NAME' => 'manufacturer_name',
			'PROD_FEATURES' => 'features'
		);
	}

	/**
	 * Get the product id, this is necessary for updating existing products.
	 *
	 * @return  int  The product ID is returned.
	 *
	 * @since   3.0
	 */
	public function getProductId()
	{
		// Find what the update is based on
		$update_based_on = $this->template->get('update_based_on', 'product_sku');

		switch ($update_based_on)
		{
			case 'product_child_sku':
				$product_sku = $this->fields->get('product_sku');
				$product_parent_sku = $this->fields->get('product_parent_sku');

				if ($product_sku && $product_parent_sku)
				{
					// Load the product parent ID
					$query = $this->db->getQuery(true)
						->select($this->db->quoteName('virtuemart_product_id'))
						->from($this->db->quoteName('#__virtuemart_products'))
						->where($this->db->quoteName('product_sku') . ' = ' . $this->db->quote($product_parent_sku));
					$this->db->setQuery($query);
					$this->log->add(JText::_('COM_CSVI_FIND_PRODUCT_CHILD_PARENT_SKU'), true);
					$product_parent_id = $this->db->loadResult();

					// Load the product ID of the child
					$query = $this->db->getQuery(true)
						->select($this->db->quoteName('virtuemart_product_id'))
						->from($this->db->quoteName('#__virtuemart_products'))
						->where($this->db->quoteName('product_sku') . ' = ' . $this->db->quote($product_sku))
						->where($this->db->quoteName('product_parent_id') . ' = ' . (int) $product_parent_id);
					$this->db->setQuery($query);
					$this->log->add(JText::_('COM_CSVI_FIND_PRODUCT_CHILD_SKU'), true);

					return $this->db->loadResult();
				}
				elseif ($product_sku)
				{
					$query = $this->db->getQuery(true)
						->select($this->db->quoteName('virtuemart_product_id'))
						->from($this->db->quoteName('#__virtuemart_products'))
						->where($this->db->quoteName('product_sku') . ' = ' . $this->db->quote($product_sku));
					$this->db->setQuery($query);
					$this->log->add(JText::_('COM_CSVI_FIND_PRODUCT_SKU_BASED_CHILD'), true);

					return $this->db->loadResult();
				}
				else
				{
					$this->log->add(JText::_('COM_CSVI_NO_CHILD_NO_PARENT'));

					return false;
				}
				break;
			default:
				$update_based_on_value = $this->fields->get($update_based_on);

				if ($update_based_on_value)
				{
					$query = $this->db->getQuery(true)
						->select($this->db->quoteName('virtuemart_product_id'))
						->from($this->db->quoteName('#__virtuemart_products'))
						->where($this->db->quoteName($update_based_on) . ' = ' . $this->db->quote($update_based_on_value));
					$this->db->setQuery($query);
					$this->log->add('Found updated based on field ' . $update_based_on);

					return $this->db->loadResult();
				}
				else
				{
					$this->log->add('Update based on value not found', false);

					return false;
				}
				break;
		}
	}

	/**
	 * Get the shopper group id.
	 *
	 * Only get the shopper group id when the shopper_group_name is set.
	 *
	 * @param   string  $shopper_group_name  The name of the shopper group to find.
	 *
	 * @return  int  The shopper group ID.
	 *
	 * @since   3.0
	 */
	public function getShopperGroupId($shopper_group_name)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('virtuemart_shoppergroup_id'))
			->from($this->db->quoteName('#__virtuemart_shoppergroups'))
			->where($this->db->quoteName('shopper_group_name') . ' = ' . $this->db->quote($shopper_group_name));
		$this->db->setQuery($query);
		$shopper_group_id = $this->db->loadResult();
		$this->log->add('Get the shopper group ID');

		return $shopper_group_id;
	}

	/**
	 * Get the shopper group id.
	 *
	 * Only get the shopper group id when the shopper_group_name is set.
	 *
	 * @param   string  $manufacturer_name  The name of the shopper group to find.
	 *
	 * @return  int  The shopper group ID.
	 *
	 * @since   3.0
	 */
	public function getManufacturerId($manufacturer_name)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('virtuemart_manufacturer_id'))
			->from($this->db->quoteName('#__virtuemart_manufacturers_' . $this->template->get('language')))
			->where($this->db->quoteName('mf_name') . ' = ' . $this->db->quote($manufacturer_name));
		$this->db->setQuery($query);
		$manufacturer_id = $this->db->loadResult();
		$this->log->add('Get the manufacturer ID');

		return $manufacturer_id;
	}

	/**
	 * Get the currency ID of the specified vendor.
	 *
	 * @param   int  $vendor_id  The ID of the vendor to get the currency for.
	 *
	 * @return  int  The vendor currency ID.
	 *
	 * @since   4.0
	 */
	public function getVendorCurrency($vendor_id)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('vendor_currency'))
			->from($this->db->quoteName('#__virtuemart_vendors'))
			->where($this->db->quoteName('virtuemart_vendor_id') . ' = ' . (int) $vendor_id);
		$this->db->setQuery($query);

		return $this->db->loadResult();
	}

	/**
	 * Gets the default Shopper Group ID.
	 *
	 * @return  int  The default shoppper group ID.
	 *
	 * @since   4.0
	 */
	public function getDefaultShopperGroupID()
	{
		$vendor_id = $this->getVendorId();
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('virtuemart_shoppergroup_id'))
			->from($this->db->quoteName('#__virtuemart_shoppergroups'))
			->where($this->db->quoteName('default') . ' = 1')
			->where($this->db->quoteName('virtuemart_vendor_id') . ' = ' . (int) $vendor_id);
		$this->db->setQuery($query);
		$default = $this->db->loadResult();
		$this->log->add('Get the default shopper group ID');

		return $default;
	}

	/**
	 * Determine vendor ID
	 *
	 * Determine for which vendor we are importing product details.
	 *
	 * The default vendor is the one with the lowest vendor_id value.
	 *
	 * @param   string  $vendor_name  Vendor name
	 * @param   bool    $nameOnly     Set if only the name should be checked
	 *
	 * @return  int  The vendor ID.
	 *
	 * @since   3.0
	 *
	 * @todo    Add full vendor support when VirtueMart supports it
	 */
	public function getVendorId($vendor_name = '', $nameOnly = false)
	{
		// If there is vendor name set, get the id first
		if ($vendor_name)
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('virtuemart_vendor_id', 'vendor_id'))
				->from($this->db->quoteName('#__virtuemart_vendors'))
				->where($this->db->quoteName('vendor_name') . ' = ' . $this->db->quote($vendor_name));
			$this->db->setQuery($query);
			$this->vendorId = $this->db->loadResult();
			$this->log->add('Check if the vendor exists with given vendor name');

			if ($nameOnly)
			{
				return $this->vendorId;
			}
		}

		if (!$this->vendorId)
		{
			// Get some values
			$vendor_id = $this->fields->get('virtuemart_vendor_id');
			$product_sku = $this->fields->get('product_sku', false);

			// User is uploading vendor_id
			if ($vendor_id)
			{
				return $vendor_id;
			}

			// User is not uploading vendor_id
			// First get the vendor with the lowest ID
			$query = $this->db->getQuery(true)
				->select('MIN(' . $this->db->quoteName('virtuemart_vendor_id') . ') AS vendor_id')
				->from($this->db->quoteName('#__virtuemart_vendors'));
			$this->db->setQuery($query);
			$min_vendor_id = $this->db->loadResult();

			if ($min_vendor_id)
			{
				if ($product_sku)
				{
					$query = $this->db->getQuery(true)
						->select(
							'IF (COUNT(' . $this->db->quoteName('virtuemart_vendor_id') . ') = 0, '
								. $min_vendor_id . ', '
								. $this->db->quoteName('virtuemart_vendor_id')
							. ') AS vendor_id'
						)
						->from($this->db->quoteName('#__virtuemart_products'))
						->where($this->db->quoteName('product_sku') . ' = ' . $this->db->quote($product_sku));
					$this->db->setQuery($query);

					// Existing vendor_id
					$vendor_id = $this->db->loadResult();
					$this->log->add('Check if the vendor exists');

					$this->vendorId = $vendor_id;

					return $vendor_id;
				}
				else
				{
					// No product_sku uploaded
					$this->vendorId = $min_vendor_id;

					return $min_vendor_id;
				}
			}
			else
			{
				// No vendor found, so lets default to 1
				$this->vendorId = 1;
			}
		}

		return $this->vendorId;
	}

	/**
	 * Create a slug.
	 *
	 * @param   string  $name  The string to turn into a slug
	 *
	 * @return  string  The slug created.
	 *
	 * @since   4.0
	 */
	public function createSlug($name)
	{
		// Get the correct language
		$sourcelanguage = $this->template->get('language', '', null, 0, false);
		$targetlanguage = $this->template->get('target_language', $sourcelanguage, null, 0, false);

		if ($sourcelanguage == $targetlanguage)
		{
			$uselang = $sourcelanguage;
		}
		else
		{
			$uselang = $targetlanguage;
		}

		// Transliterate
		$lang = new JLanguage($uselang);
		$str = $lang->transliterate($name);

		// Trim white spaces at beginning and end of alias and make lowercase
		$str = trim(\Joomla\String\StringHelper::strtolower($str));

		// Remove any duplicate whitespace, and ensure all characters are alphanumeric
		$str = preg_replace('/(\s|[^A-Za-z0-9\-])+/', '-', $str);

		// Trim dashes at beginning and end of alias
		$str = trim($str, '-');

		// If we are left with an empty string, make a date with random number
		if (trim(str_replace('-', '', $str)) == '')
		{
			$jdate = JFactory::getDate();
			$str = $jdate->format("Y-m-d-h-i-s") . mt_rand();
		}

		return $str;
	}

	/**
	 * Get the custom related field ID.
	 *
	 * @param   string  $type  Type of field
	 *
	 * @return  int  The ID of the related field.
	 *
	 * @since   4.0
	 */
	public function getRelatedId($type = 'R')
	{
		if (!$this->relatedId || $this->relatedType !== $type)
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('virtuemart_custom_id'))
				->from($this->db->quoteName('#__virtuemart_customs'))
				->where($this->db->quoteName('field_type') . ' = ' . $this->db->quote($type));
			$this->db->setQuery($query);
			$this->relatedId = $this->db->loadResult();
			$this->relatedType = $type;
		}

		return $this->relatedId;
	}

	/**
	 * Load the order status code.
	 *
	 * @param   string  $order_status_name  The name of the order status.
	 *
	 * @return  string  The order status code.
	 *
	 * @since   2.3.11
	 */
	public function getOrderStatus($order_status_name)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('order_status_code'))
			->from($this->db->quoteName('#__virtuemart_orderstates'))
			->where($this->db->quoteName('order_status_name') . ' = ' . $this->db->quote($order_status_name));
		$this->db->setQuery($query);

		$orderStatusCode = $this->db->loadResult();
		$this->log->add('Get the order status code');

		return $orderStatusCode;
	}

	/**
	 * Get the currency ID.
	 *
	 * @param   string  $currency_name  The name of the currency
	 * @param   int     $vendor_id      The ID of the vendor
	 *
	 * @return  int  The currency ID.
	 *
	 * @since   4.0
	 */
	public function getCurrencyId($currency_name, $vendor_id)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('virtuemart_currency_id'))
			->from($this->db->quoteName('#__virtuemart_currencies'))
			->where($this->db->quoteName('currency_code_3') . ' = ' . $this->db->quote($currency_name))
			->where($this->db->quoteName('virtuemart_vendor_id') . ' = ' . (int) $vendor_id);
		$this->db->setQuery($query);
		$currency_id = $this->db->loadResult();
		$this->log->add('Get the currency ID');

		return $currency_id;
	}

	/**
	 * Get the country ID.
	 *
	 * @param   string  $country_name    The name of the country
	 * @param   string  $country_2_code  The 2 letter notification
	 * @param   string  $country_3_code  The 3 letter notification
	 *
	 * @return  int  The ID of the country.
	 *
	 * @since   4.0
	 */
	public function getCountryId($country_name=null, $country_2_code=null, $country_3_code=null)
	{
		$country_id = null;

		if (isset($country_name) || isset($country_2_code) || isset($country_3_code))
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('virtuemart_country_id'))
				->from($this->db->quoteName('#__virtuemart_countries'));

			if (isset($country_name))
			{
				$query->where($this->db->quoteName('country_name') . ' = ' . $this->db->quote($country_name));
			}
			elseif (isset($country_2_code))
			{
				$query->where($this->db->quoteName('country_2_code') . ' = ' . $this->db->quote($country_2_code));
			}
			elseif (isset($country_3_code))
			{
				$query->where($this->db->quoteName('country_3_code') . ' = ' . $this->db->quote($country_3_code));
			}

			$this->db->setQuery($query);
			$country_id = $this->db->loadResult();
			$this->log->add('Get the country ID', true);
		}

		return $country_id;
	}

	/**
	 * Get the state ID.
	 *
	 * @param   string  $state_name	   The name of the state
	 * @param   string  $state_2_code  The 2 letter notification
	 * @param   string  $state_3_code  The 3 letter notification
	 * @param   int     $countryId     The ID of the country the state belongs to
	 *
	 * @return  int  The state ID.
	 *
	 * @since   4.0
	 */
	public function getStateId($state_name=null, $state_2_code=null, $state_3_code=null, $countryId=null)
	{
		$state_id = null;

		if (isset($state_name) || isset($state_2_code) || isset($state_3_code))
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('virtuemart_state_id'))
				->from($this->db->quoteName('#__virtuemart_states'))
				->where($this->db->quoteName('virtuemart_country_id') . ' = ' . (int) $countryId);

			if (isset($state_name))
			{
				$query->where($this->db->quoteName('state_name') . ' = ' . $this->db->quote($state_name));
			}
			elseif (isset($state_2_code))
			{
				$query->where($this->db->quoteName('state_2_code') . ' = ' . $this->db->quote($state_2_code));
			}
			elseif (isset($state_3_code))
			{
				$query->where($this->db->quoteName('state_3_code') . ' = ' . $this->db->quote($state_3_code));
			}

			$this->db->setQuery($query);
			$state_id = $this->db->loadResult();
			$this->log->add('Get the state ID', true);
		}

		return $state_id;
	}

	/**
	 * Get category tree.
	 *
	 * @param   string  $language  The language code for the category names.
	 *
	 * @return  array  The list of select options.
	 *
	 * @since   4.0
	 */
	public function getCategoryTree($language)
	{
		// Clean up the language if needed
		$language = strtolower(str_replace('-', '_', $language));

		$query = $this->db->getQuery(true);

		// 1. Get all categories
		$query->select('x.category_parent_id AS parent_id, x.category_child_id AS id, l.category_name AS catname');
		$query->from('#__virtuemart_categories c');
		$query->leftJoin('#__virtuemart_category_categories x ON c.virtuemart_category_id = x.category_child_id');
		$query->leftJoin('#__virtuemart_categories_' . $language . ' l ON l.virtuemart_category_id = c.virtuemart_category_id');
		$this->db->setQuery($query);
		$rawcats = $this->db->loadObjectList();

		if ($rawcats)
		{
			// 2. Group categories based on their parent_id
			$categories = array();

			foreach ($rawcats as $rawcat)
			{
				$categories[$rawcat->parent_id][$rawcat->id]['pid'] = $rawcat->parent_id;
				$categories[$rawcat->parent_id][$rawcat->id]['cid'] = $rawcat->id;
				$categories[$rawcat->parent_id][$rawcat->id]['catname'] = $rawcat->catname;
			}
		}

		// Free up some memory
		unset($rawcats);

		// Clean the array
		$this->selectOptions = array();

		// Add a don't use option
		$this->selectOptions[] = JHtml::_('select.option', '', JText::_('COM_CSVI_EXPORT_DONT_USE'));

		if (isset($categories))
		{
			if (count($categories) > 0)
			{
				// Take the toplevels first
				foreach ($categories[0] as $key => $category)
				{
					$this->selectOptions[] = JHtml::_('select.option', $category['cid'], $category['catname']);

					// Write the subcategories
					$suboptions = $this->buildCategory($categories, $category['cid'], array());
				}
			}
		}

		// Free up some memory
		unset($categories);

		// Return the select options
		return $this->selectOptions;
	}

	/**
	* Create the subcategory layout
	*
	* @copyright
	* @author		RolandD
	* @todo
	* @see
	* @access 		private
	* @param
	* @return 		array	select options for the category tree
	* @since 		3.0
	*/
	private function buildCategory($cattree, $catfilter, $subcats, $loop=1) {
		if (isset($cattree[$catfilter])) {
			foreach ($cattree[$catfilter] as $subcatid => $category) {
				$this->selectOptions[] = JHtml::_('select.option', $category['cid'], str_repeat('>', $loop).' '.$category['catname']);
				$subcats = $this->buildCategory($cattree, $subcatid, $subcats, $loop+1);
			}
		}
	}

	/**
	 * Creates the category path based on a product ID.
	 *
	 * @param   int   $product_id  The product ID to create the category path for.
	 * @param   bool  $id          Set if only the IDs should be returned
	 *
	 * @return  string  The category path.
	 *
	 * @since   3.0
	 */
	public function createCategoryPath($product_id, $id=false)
	{
		// Get the category paths
		$query = $this->db->getQuery(true);
		$query->select($this->db->quoteName('virtuemart_category_id'));
		$query->from($this->db->quoteName('#__virtuemart_product_categories'));
		$query->where($this->db->quoteName('virtuemart_product_id') . ' = ' . $this->db->quote($product_id));
		$this->db->setQuery($query);
		$catids = $this->db->loadColumn();

		if (!empty($catids))
		{
			// Return the paths
			if ($id)
			{
				$result = $this->db->loadColumn();

				if (is_array($result))
				{
					return implode('|', $result);
				}
				else
				{
					return null;
				}
			}
			else
			{
				$catpaths = $this->constructCategoryPath($catids);

				if (is_array($catpaths))
				{
					return implode('|', $catpaths);
				}
				else
				{
					return null;
				}
			}
		}
		else
		{
			return null;
		}
	}

	/**
	 * Construct the category path.
	 *
	 * @param   array   $catids    The IDs to build a category for.
	 * @param   string  $language  The name of the language selector.
	 *
	 * @return  array  List of category paths.
	 *
	 * @since   4.0
	 */
	private function  constructCategoryPath($catids, $language='language')
	{
		$catpaths = array();

		if (is_array($catids))
		{
			// Load the category separator
			if (is_null($this->categorySeparator))
			{
				$this->categorySeparator = $this->template->get('category_separator', '/');
			}

			// Get the paths
			foreach ($catids as $category_id)
			{
				// Create the path
				$paths = array();

				while ($category_id > 0)
				{
					if (!array_key_exists($category_id, $this->categories))
					{
						$query = $this->db->getQuery(true)
							->select(
								$this->db->quoteName(
									[
										'x.category_parent_id',
										'l.category_name'
									]
								)
							)
							->from($this->db->quoteName('#__virtuemart_category_categories', 'x'))
							->leftJoin(
								$this->db->quoteName('#__virtuemart_categories', 'c')
								. ' ON ' . $this->db->quoteName('x.category_child_id') . ' = ' . $this->db->quoteName('c.virtuemart_category_id')
							)
							->leftJoin(
								$this->db->quoteName('#__virtuemart_categories_' . $this->template->get($language), 'l')
								. ' ON ' . $this->db->quoteName('x.category_child_id') . ' = ' . $this->db->quoteName('l.virtuemart_category_id')
							)
							->where($this->db->quoteName('category_child_id') . ' = ' . (int) $category_id);
						$this->db->setQuery($query);
						$path = $this->db->loadObject();
						$this->log->add('Get cat ID' . $category_id);

						$this->categories[$category_id] = $path;
					}

					$path = $this->categories[$category_id];

					if (is_object($path))
					{
						$paths[] = $path->category_name;
						$category_id = $path->category_parent_id;
					}
					else
					{
						$this->log->add('COM_CSVI_CANNOT_GET_CATEGORY_ID');

						return '';
					}
				}

				// Create the path
				$paths = array_reverse($paths);
				$catpaths[] = implode($this->categorySeparator, $paths);
			}
		}

		return $catpaths;
	}

	/**
	 * Create a category path based on ID.
	 *
	 * @param   array   $catids    List of IDs to generate category path for.
	 * @param   string  $language  The name of the language selector.
	 *
	 * @return  string  The constructed path.
	 *
	 * @since   4.0
	 */
	public function createCategoryPathById($catids, $language='language')
	{
		if (!is_array($catids))
		{
			$catids = (array) $catids;
		}

		$paths = $this->constructCategoryPath($catids, $language);

		if (is_array($paths))
		{
			return implode('|', $paths);
		}
		else
		{
			return '';
		}
	}

	/**
	 * Get the category ID for a product.
	 *
	 * @param   int  $product_id  The product ID to get the category for
	 *
	 * @return  int  The category ID the product is linked to limited to 1.
	 *
	 * @since   3.0
	 */

	public function getCategoryId($product_id)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('virtuemart_category_id'))
			->from($this->db->quoteName('#__virtuemart_product_categories'))
			->where($this->db->quoteName('virtuemart_product_id') . ' = ' . (int) $product_id);
		$this->db->setQuery($query, 0, 1);

		return $this->db->loadResult();
	}

	/**
	 * Determine the shipping cost.
	 *
	 * @return  mixed  Prices if available | False otherwise.
	 *
	 * @since
	 */
	public function shippingCost($product_price)
	{
		$prices = $this->template->get('shopper_shipping_export_fields', array());
		$fee = null;

		if (!empty($prices))
		{
			foreach ($prices['_price_from'] as $kfrom => $price_from)
			{
				// Check if we have an end price
				$price_from = str_replace(',', '.', $price_from);
				$price_to = str_replace(',', '.', $prices['_price_to'][$kfrom]);

				if (!empty($price_to))
				{
					if ($product_price >= $price_from && $product_price < $price_to)
					{
						$fee = $kfrom;
						break;
					}
				}
				else
				{
					if ($product_price >= $price_from)
					{
						$fee = $kfrom;
						break;
					}
				}
			}
		}

		if (!is_null($fee))
		{
			return $prices['_fee'][$fee];
		}
		else
		{
			return false;
		}
	}

	/**
	* Get the list of order products
	*
	* @copyright
	* @author		RolandD
	* @todo
	* @see
	* @access 		public
	* @param
	* @return 		array of objects
	* @since 		4.0
	*/
	public function getOrderProduct() {
		$jinput = JFactory::getApplication()->input;
		$db = JFactory::getDbo();
		$query = $this->db->getQuery(true);
		$filter = $jinput->get('filter');
		$q = "SELECT DISTINCT order_item_sku AS product_sku, order_item_name AS product_name
				FROM #__virtuemart_order_items o
				WHERE (order_item_sku LIKE ".$this->db->Quote('%'.$filter.'%')."
					OR order_item_name LIKE ".$this->db->Quote('%'.$filter.'%').")
				ORDER BY order_item_name
				LIMIT 10;";
		$this->db->setQuery($q);
		return $this->db->loadObjectList();
	}

	/**
	 * Get the list of order item products
	 *
	 * @copyright
	 * @author		RolandD
	 * @todo
	 * @see
	 * @access 		public
	 * @param
	 * @return 		array of objects
	 * @since 		4.0
	 */
	public function getOrderItemProduct() {
		$jinput = JFactory::getApplication()->input;
		$filter = $jinput->get('filter');
		$q = "SELECT DISTINCT order_item_sku AS product_sku, order_item_name AS product_name
				FROM #__virtuemart_order_items o
				WHERE (o.order_item_sku LIKE ".$this->db->quote('%'.$filter.'%')."
					OR o.order_item_name LIKE ".$this->db->quote('%'.$filter.'%').")
				ORDER BY order_item_name
				LIMIT 10;";
		$this->db->setQuery($q);
		return $this->db->loadObjectList();
	}

	/**
	 * Get manufacturer list
	 *
	 * @copyright
	 * @author 		RolandD
	 * @todo
	 * @see
	 * @access 		public
	 * @param		string	$language	the language code for the category names
	 * @return
	 * @since 		4.0
	 */
	public function getManufacturers($language) {
		// Clean up the language if needed
		$lang = strtolower(str_replace('-', '_', $language));

		$db = JFactory::getDbo();
		$query = $this->db->getQuery(true);
		$query->select($this->db->quoteName('virtuemart_manufacturer_id').' AS value,'.$this->db->quoteName('mf_name').' AS text');
		$query->from($this->db->quoteName('#__virtuemart_manufacturers_'.$lang));
		$this->db->setQuery($query);
		$mfs = $this->db->loadObjectList();

		$options = array();
		// Add a don't use option
		$options[] = JHtml::_('select.option', 'none', JText::_('COM_CSVI_EXPORT_DONT_USE'));

		if (isset($mfs)) {
			if (count($mfs) > 0) {
				// Take the toplevels first
				foreach ($mfs as $key => $mf) {
					$options[] = JHtml::_('select.option', $mf->value, $mf->text);
				}
			}
		}
		return $options;
	}

	/**
	 * Unpublish products before import.
	 *
	 * @param  CsviHelperTemplate  $template  An instance of CsviHelperTemplate
	 * @param  CsviHelperLog       $log       An instance of CsviHelperLog
	 * @param  JDatabase           $db        JDatabase class
	 *
	 * @return  void.
	 *
	 * @since   3.0
	 *
	 * @throws  RuntimeException
	 */
	public function unpublishBeforeImport(CsviHelperTemplate $template, CsviHelperLog $log, JDatabase $db)
	{
		if ($this->template->get('unpublish_before_import', 0))
		{
			$table = '#__virtuemart_products';

			if ($this->template->get('operation') === 'category')
			{
				$table = '#__virtuemart_categories';
			}

			$query = $this->db->getQuery(true)
				->update($this->db->quoteName($table))
				->set($this->db->quoteName('published') . ' = 0');
			$this->db->setQuery($query);

			if ($this->db->execute())
			{
				$log->add('Unpublishing products before import');
			}
			else
			{
				$log->add('Cannot unpublish products before import');
			}
		}
	}

	/**
	 * Find the product ID based on SKU.
	 *
	 * @param   string  $productSku  The SKU to find the product ID for
	 *
	 * @return  int  The ID of the product SKU.
	 *
	 * @since   6.0
	 *
	 * @throws  RuntimeException
	 */
	public function getProductIdBySku($productSku)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('virtuemart_product_id'))
			->from($this->db->quoteName('#__virtuemart_products'))
			->where($this->db->quoteName('product_sku') . ' = ' . $this->db->quote($productSku));
		$this->db->setQuery($query);
		$this->log->add('Find the product ID by SKU');

		return $this->db->loadResult();
	}

	/**
	 * Get a list of subcategory IDs for a given parent category ID.
	 *
	 * @param   int  $category_id  The category ID to get the subcategories for.
	 *
	 * @return  array  List of subcategory IDs.
	 *
	 * @since   3.0
	 */
	public function getSubCategoryIds($category_id)
	{
		$subcats = $this->getChildren($category_id);

		if (!empty($subcats))
		{
			foreach ($subcats as $subcat)
			{
				$newcats = $this->getSubCategoryIds($subcat);

				$subcats = array_merge($subcats, $newcats);
			}
		}

		return $subcats;

	}

	private function getChildren($parent_category_id)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('x.category_child_id', 'id'))
			->from($this->db->quoteName('#__virtuemart_categories', 'c'))
			->leftJoin(
				$this->db->quoteName('#__virtuemart_category_categories', 'x')
				. ' ON ' . $this->db->quoteName('c.virtuemart_category_id') . ' = ' . $this->db->quoteName('x.category_child_id')
			)
			->where($this->db->quoteName('x.category_parent_id') . ' = ' . (int) $parent_category_id);

		$this->db->setQuery($query);

		return $this->db->loadColumn();
	}

	/**
	 * Load the ICEcat data for a product.
	 *
	 * @return  mixed  True on success | false on failure.
	 *
	 * @since   3.0
	 */
	public function getIcecat()
	{
		if ($this->template->get('use_icecat'))
		{
			// Load the ICEcat helper
			if (is_null($this->icecat))
			{
				$this->icecat = new CsviHelperIcecat($this->template, $this->log, $this->db);
			}

			// Clean up any previous ICEcat data
			$this->fields->clearIcecatData();
			$fields = array();

			// Check conditions
			// 1. Do we have an MPN
			$update_based_on = $this->template->get('update_based_on');
			$mpn = $this->fields->get($update_based_on);

			// Exclude product_child_sku field for icecat import
			if ($mpn != 'product_child_sku')
			{
				$this->log->add('Found ICEcat mpn reference: ' . $mpn, false);

				// 2. Do we have a manufacturer name for VirtueMart
				$mf_name = $this->fields->get('manufacturer_name');
				$this->log->add('Found ICEcat manufacturer name: ' . $mf_name, false);

				if ($mf_name)
				{
					// Get the data from ICEcat
					$data = $this->icecat->getData($mpn, $mf_name);

					if ($data)
					{
						// Get the list of fields not to populate with ICEcat data
						$update_fields = $this->template->get('icecat_update_fields', array(), 'array');

						// Map the ICEcat data to VirtueMart fields
						foreach ($data as $name => $value)
						{
							if (isset($this->icecatFields[$name]))
							{
								if (!in_array($this->icecatFields[$name], $update_fields))
								{
									$fields[$this->icecatFields[$name]] = $value;
								}
							}
						}

						// Add the ICEcat data to the fields
						$this->fields->setIcecatData($fields);
					}

					return true;
				}
				else
				{
					$this->log->add('ICEcat manufacturer not found', false);

					return false;
				}
			}
			else
			{
				$this->log->add('ICEcat mpn reference not found', false);

				return false;
			}
		}

		return false;
	}

	/**
	 * Get default language in virtuemart
	 *
	 * @return  string language code
	 *
	 * @since   6.5.7
	 */
	public function getDefaultLanguage()
	{
		require_once JPATH_PLUGINS . '/csviaddon/virtuemart/com_virtuemart/helper/com_virtuemart_config.php';

		$helperConfig = new Com_VirtuemartHelperCom_Virtuemart_Config;

		$language     = $helperConfig->get('active_languages');
		$languageCode = '';

		if (is_array($language) && array_key_exists(0, $language))
		{
			$languageCode = strtolower(str_replace('-', '_', $language[0]));
		}

		// Check if no language is set in VirtueMart, take the default Joomla! frontend language
		if ('' === $languageCode)
		{
			$languageCode = strtolower(str_replace('-', '_', JComponentHelper::getParams('com_languages')->get('site')));
		}

		return $languageCode;
	}

	/**
	 * Load specific language files if needed.
	 *
	 * @return  array  List of language files to load.
	 *
	 * @since   7.1.0
	 */
	public function loadLanguage()
	{
		return array(
			'com_virtuemart_shoppers' => JPATH_SITE . '/components/com_virtuemart',
			'com_virtuemart_orders' => JPATH_SITE . '/components/com_virtuemart',
		);
	}

	/**
	 * Get the order id from order number
	 *
	 * @param   string  $orderNumber  The order number to find the ID for
	 *
	 * @return  int  The order ID is returned.
	 *
	 * @since   7.3.0
	 */
	public function getOrderId($orderNumber = null)
	{
		if (!$orderNumber)
		{
			return false;
		}

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('virtuemart_order_id'))
			->from($this->db->quoteName('#__virtuemart_orders'))
			->where($this->db->quoteName('order_number') . ' = ' . $this->db->quote($orderNumber));
		$this->db->setQuery($query);
		$this->log->add('Found order id for the given order number');

		return $this->db->loadResult();
	}

	/**
	 * Get the custom field group ID.
	 *
	 * @param   string  $groupName  The group name.
	 *
	 * @return  int  The ID of the group.
	 *
	 * @since   7.3.0
	 *
	 * @throws  RuntimeException
	 */
	public function getCustomFieldGroup($groupName)
	{
		if (!$groupName)
		{
			return false;
		}

		$query = $this->db->getQuery(true);
		$query->select('virtuemart_custom_id')
			->from($this->db->quoteName('#__virtuemart_customs'))
			->where($this->db->quoteName('custom_title') . '  = ' . $this->db->quote($groupName));
		$this->db->setQuery($query);
		$groupId = $this->db->loadResult();

		if (!$groupId)
		{
			$this->log->add('No group found with name ' . $groupName);
		}

		return $groupId;
	}

	/**
	 * Get a active virtuemart languages
	 *
	 * @return  array.
	 *
	 * @since   7.5.0
	 *
	 */
	public function getActiveLanguages()
	{
		require_once JPATH_PLUGINS . '/csviaddon/virtuemart/com_virtuemart/helper/com_virtuemart_config.php';
		$helperConfig = new \Com_VirtuemartHelperCom_Virtuemart_Config;

		return $helperConfig->get('active_languages');
	}

	/**
	 * Process Product Attribute field for export
	 *
	 * @param   string $fieldValue The Field Value.
	 *
	 * @return  mixed.
	 *
	 * @since   7.11.0
	 *
	 */
	public function processProductAttribute($fieldValue)
	{
		if (!$fieldValue)
		{
			return false;
		}

		$options    = json_decode($fieldValue, true);
		$fieldTypes = array_keys($options);
		$values     = array();

		if ($fieldTypes)
		{
			foreach ($fieldTypes as $ftype)
			{
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName(array('field_type', 'custom_title')))
					->from($this->db->quoteName('#__virtuemart_customs'))
					->where($this->db->quoteName('virtuemart_custom_id') . ' = ' . (int) $ftype);

				$this->db->setQuery($query);
				$fieldType = $this->db->loadRow();

				switch ($fieldType[0])
				{
					case 'S':
					case 'M':
						$customFieldId = $options[$ftype];

						// Check if the string custom field is predefined list
						if (is_array($options[$ftype]))
						{
							$query->clear()
								->select($this->db->quoteName('custom_value'))
								->from($this->db->quoteName('#__virtuemart_customs'))
								->where($this->db->quoteName('is_list') . ' = 1')
								->where($this->db->quoteName('virtuemart_custom_id') . ' = ' . (int) $ftype);
							$this->db->setQuery($query);
							$listDefinedValues = $this->db->loadResult();

							if ($listDefinedValues)
							{
								$stringKey   = array_keys($options[$ftype]);
								$stringValue = $options[$ftype][$stringKey[0]];
								$values[]    = \JText::_($fieldType[1]) . ' - ' . $stringValue;
							}
						}
						else
						{
							$query->clear()
								->select($this->db->quoteName('customfield_value'))
								->from($this->db->quoteName('#__virtuemart_product_customfields'))
								->where($this->db->quoteName('virtuemart_customfield_id') . ' = ' . (int) $customFieldId);
							$this->db->setQuery($query);
							$customFieldStoredValue = $this->db->loadResult();
							$values[]               = \JText::_($fieldType[1]) . ' - ' . $customFieldStoredValue;
						}

						break;
					case 'D':
						$customFieldDate = $options[$ftype];
						$values[]        = \JText::_($fieldType[1]) . ' - ' . $customFieldDate[$ftype];
						break;
					case 'E':
						// Load the plugins
						$dispatcher = new \RantaiPluginDispatcher;
						$dispatcher->importPlugins('csviext', $this->db);
						$values[] = $dispatcher->trigger(
							'exportProductAttribute',
							array(
								'options'   => $options[$ftype],
								'fieldType' => $fieldType,
								'ftype'     => $ftype
							)
						);

						break;
					default:
						break;
				}
			}
		}

		return $values;
	}

	/**
	 * Get a list of shopper custom fields
	 *
	 * @param   int  $coreFields  If it is core fields or not
	 *
	 * @return  array Shopper fields array.
	 *
	 * @since   7.10.1
	 */
	public function loadShopperCustomFields($coreFields = 0)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('name'))
			->from($this->db->quoteName('#__virtuemart_userfields'))
			->where($this->db->quoteName('sys') . ' =  ' . (int) $coreFields);
		$this->db->setQuery($query);

		return $this->db->loadColumn();
	}

	/**
	 * Get a list of calc names for VatTax.
	 *
	 * @return  array calc names array.
	 *
	 * @since   6.5.0
	 */
	public function loadCalcNames()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('calc_name'))
			->from($this->db->quoteName('#__virtuemart_calcs'))
			->where($this->db->quoteName('calc_kind') . ' = ' . $this->db->quote('VatTax'));
		$this->db->setQuery($query);

		$result = $this->db->loadColumn();

		if (!is_array($result))
		{
			$result = array();
		}

		 return $result;
	}

	/**
	 * Get a list of custom fields that can be used as available field.
	 *
	 * @return  array Array of custom fields.
	 *
	 * @since   7.3.0
	 *
	 * @throws  \RuntimeException
	 */
	public function loadCustomFields()
	{
		$query = $this->db->getQuery(true);
		$query->select('TRIM(' . $this->db->quoteName('custom_title') . ') AS ' . $this->db->quoteName('title'));
		$query->from($this->db->quoteName('#__virtuemart_customs'));
		$query->where(
			$this->db->quoteName('field_type') . ' IN ('
			. $this->db->quote('S') . ','
			. $this->db->quote('I') . ','
			. $this->db->quote('B') . ','
			. $this->db->quote('D') . ','
			. $this->db->quote('T') . ','
			. $this->db->quote('M') . ','
			. $this->db->quote('Y') . ','
			. $this->db->quote('X') .
			')'
		);
		$this->db->setQuery($query);
		$result = $this->db->loadColumn();

		if (!is_array($result))
		{
			$result = array();
		}

		return $result;
	}

	/**
	 * Get the value of tax field.
	 *
	 * @param   string $calc_name The name of the tax calculated.
	 * @param   string $order_id  The order id of the tax amount.
	 *
	 * @return  float Calculated Tax value.
	 *
	 * @since   6.5.0
	 */
	public function getOrderBillTaxAmount($calc_name, $order_id)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('order_billTax'))
			->from($this->db->quoteName('#__virtuemart_orders'))
			->where($this->db->quoteName('virtuemart_order_id') . ' = ' . (int) $order_id);
		$this->db->setQuery($query);
		$result = $this->db->loadRow();

		$taxValue = 0;

		if (isset($result[0]))
		{
			$taxAmount = json_decode($result[0]);

			if (!empty($taxAmount))
			{
				foreach ($taxAmount as $tax)
				{
					if ($calc_name == $tax->calc_name)
					{
						$taxValue = $tax->result;
					}
				}
			}
		}

		return $taxValue;
	}

	/**
	 * Get the canonical category ID for a product.
	 *
	 * @param   int  $productId  The product ID to get the category for
	 *
	 * @return  int  The category ID the product is linked to limited to 1.
	 *
	 * @since   7.15.0
	 */
	public function getCanonicalCategoryId($productId)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('product_canon_category_id'))
			->from($this->db->quoteName('#__virtuemart_products'))
			->where($this->db->quoteName('virtuemart_product_id') . ' = ' . (int) $productId);
		$this->db->setQuery($query);

		return $this->db->loadResult();
	}

	/**
	 * Check if the field exists in the table
	 *
	 * @param   string  $tableName  The table name
	 * @param   string  $fieldName  The field name
	 *
	 * @return  bool True if field exists false otherwise.
	 *
	 * @since   7.17.0
	 */
	public function checkHasLinkedField($tableName, $fieldName)
	{
		$existingFields = $this->db->getTableColumns($tableName, false);

		if (array_key_exists($fieldName, $existingFields))
		{
			return true;
		}

		return false;
	}
}
